import sys
from jsonschema import validate, RefResolver, Draft202012Validator
import json
from pathlib import Path
import logging
import argparse

def handle_uri(u):
    print(u)

def load_json(path : str):
    with open(path) as f:
        return json.load(f)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()

    parser.add_argument('-s','--schemadir', action="append", help="Directory containing supporting schema files to use for validation")
    parser.add_argument('-v', '--verbose', action="count", help="Verbose logging (can be specified multiple times)")
    parser.add_argument('-i', '--input', type=argparse.FileType('r'), default=sys.stdin, help="Path to input file (if absent, stdin is used)")

    args = parser.parse_args()

    match args.verbose:
        case v if v and v >= 2:
            logging.basicConfig(level=logging.DEBUG)
        case 1:
            logging.basicConfig(level=logging.INFO)
        case _:
            logging.basicConfig(level=logging.WARNING)

    logging.debug(f"Arguments: {args}")

    instance_doc = json.loads(args.input.read())
    args.input.close()

    config = {
        'schema_include_dirs' : [
            '../schema/',
            '../../103280/',
        ],
        'main_schema_doc' : '../schema/response.schema.json'
    }

    rootPath = Path(sys.argv[0]).parent
    main_schema = load_json(str(rootPath / config['main_schema_doc']))
    schema_dict = { main_schema['$id'] : main_schema }

    schema_paths = []    
    for d in config['schema_include_dirs']:
        schema_paths += [f for f in (rootPath / Path(d)).rglob("*.schema.json")]
    logging.info(f"Core schema files loaded: {schema_paths}")
    if args.schemadir:
        for d in args.schemadir:
            schema_paths += [f for f in Path(d).rglob("*.schema.json")]
        logging.info(f"CSP schema files loaded: {schema_paths}")
    else:
        logging.info(f"No CSP schema files loaded")
    schemas_json = [json.load(p.open()) for p in schema_paths]
    schema_dict = schema_dict | { s['$id'] : s for s in schemas_json }

    logging.info(f"Schema IDs loaded: {[k for k in schema_dict.keys()]}")

    logging.debug (f"Instance doc: {instance_doc}")
    logging.debug (f"Main schema: {main_schema}")

    resolver = RefResolver(None, 
                           referrer=None, 
                           store=schema_dict)
    
    logging.info("Performing ETSI validation")
    v = Draft202012Validator(main_schema, resolver=resolver)
    v.validate(instance_doc)

    logging.info("Building record type dictionary")
    type_dict = instance_doc['recordSetDescription']['recordTypes']
    logging.debug(type_dict)
    ref_dict = { k : {"$ref" : v} for k,v in type_dict.items()}
    validator_dict = { k : Draft202012Validator(ref_dict[k], resolver=resolver) for k,v in ref_dict.items()}
    logging.debug(ref_dict)

    logging.info("Validating records")
    for r in instance_doc['recordSet']:
        type_key = r['type']
        if type_key not in type_dict.keys():
            logging.error(f"Record {r['id']} has type {type_key}, not in recordType dict")
        type_ref = type_dict[type_key]
        type_schema_id = type_ref.split('#')[0]
        logging.info(f"Using {type_schema_id} to validate {type_ref} in record {r['id']}")
        if not (type_key in validator_dict.keys()):
            logging.error(f'Type key {type_key} from type {type_ref} in record {r["id"]} not in validator dictionary')
            print(ref_dict)
        v = validator_dict[type_key]
        v.validate(r)

    logging.info("Done")